/**************************************************************************
 *    file:         vtWave.h                                              *
 *    create:       June 9, 2000                                          *
 *    copyright:    @ 2000, by Zhaohui Xing                               *
 *    e-mail:       xinghh@yahoo.com                                      * 
 *    description:  definition of the sound process class                 *
 **************************************************************************/

#ifndef __VTWAVE_H__
#define __VTWAVE_H__
#include <mmsystem.h>
#include "vtDef.h"

/**************************************************************************
 *  The base class for wave format data I/O                               *
 **************************************************************************/
class VTWave
{
public:
    VTWave(HWND hwnd = NULL, int iFormat = VT_WAVE1M08)
	{
        m_hWnd = hwnd;
	    m_iFormat = iFormat;
	};

public:
    void SetHwnd(HWND hwnd){m_hWnd = hwnd;};
    void SetVTWaveFormat(int iFormat){m_iFormat = iFormat;};
	int  GetVTWaveFormat(void){return m_iFormat;};
  
    virtual BOOL Initialize(void){return TRUE;};

protected:
    HWND        m_hWnd;           // The windows handle which process sound
	                              // I/O
    int         m_iFormat;        // Wave data format. The format type has 
	                              // been defined 
};


/**************************************************************************
 *  The class for play sound                                              *
 **************************************************************************/
class VTPlayWave : public VTWave
{
public:
    VTPlayWave();
	VTPlayWave(HWND hwnd, int iFormat);
    ~VTPlayWave();

    BOOL AllocMemory(void);
	void CleanMemory(void);
    BOOL Initialize(void);
    BOOL StartPlay(void);
	void StopPlay(void);
	void ResetPlay(void);
	BOOL RestartPlay(void);
	void SetPlayData(void* pWaveData, unsigned int lBufSize);

private:
    HWAVEOUT    m_hwOut;          // Handle of wave output device   
    WAVEHDR*    m_pWhdrOut;       // Wave header for wave output

};


/**************************************************************************
 *  The class for record sound                                            *
 **************************************************************************/
class VTRecordWave : public VTWave
{
public:
    VTRecordWave();
	VTRecordWave(HWND hwnd, int iFormat);
    ~VTRecordWave();

    BOOL AllocMemory(void);
	void CleanMemory(void);
    BOOL Initialize(void);
	BOOL StartRecord(void);
	void StopRecord(void);
	BOOL ResetRecord(void);

private:
	HWAVEIN     m_hwIn;          // Handle of wave input device
    WAVEHDR*    m_pWhdrIn;       // Wave header for wave input
};



#endif